#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: List cards introduced in the system

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys

from smartcard.scard import (
    SCARD_S_SUCCESS,
    SCARD_SCOPE_USER,
    SCardEstablishContext,
    SCardGetErrorMessage,
    SCardReleaseContext,
    error,
    resourceManager,
)

if "winscard" == resourceManager:

    # pylint: disable=no-name-in-module
    from smartcard.scard import (
        SCARD_PROVIDER_CSP,
        SCARD_PROVIDER_PRIMARY,
        SCardGetCardTypeProviderName,
        SCardListCards,
    )

    # Cryptoflex 8k v2 is introduced in standard Windows 2000
    slbCryptoFlex8kv2ATR = [0x3B, 0x95, 0x15, 0x40, 0x00, 0x68, 0x01, 0x02, 0x00, 0x00]
    try:
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to establish context: " + SCardGetErrorMessage(hresult))
        print("Context established!")

        try:
            hresult, card = SCardListCards(hcontext, slbCryptoFlex8kv2ATR, [])
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failure to locate Schlumberger Cryptoflex 8k v2 card: "
                    + SCardGetErrorMessage(hresult)
                )
            print("Located by ATR:", card)

            hresult, cards = SCardListCards(hcontext, [], [])
            if hresult != SCARD_S_SUCCESS:
                raise error("Failure to list cards: " + SCardGetErrorMessage(hresult))
            print("Cards:", cards)

            for i in cards:
                hresult, providerguid = SCardGetCardTypeProviderName(
                    hcontext, i, SCARD_PROVIDER_PRIMARY
                )
                if hresult == SCARD_S_SUCCESS:
                    print(i, "Primary provider:", providerguid)
                hresult, providername = SCardGetCardTypeProviderName(
                    hcontext, i, SCARD_PROVIDER_CSP
                )
                if hresult == SCARD_S_SUCCESS:
                    print(i, "CSP Provider:", providername)

        finally:
            hresult = SCardReleaseContext(hcontext)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failed to release context: " + SCardGetErrorMessage(hresult)
                )
            print("Released context.")

    except error as e:
        print(e)

elif "pcsclite" == resourceManager:
    print("SCardListCards not supported by pcsc lite")


if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)



# pylint: disable=missing-module-docstring
# pylint: disable=invalid-name
# pylint: disable=missing-function-docstring

from smartcard.scard import (
    SCARD_E_NO_READERS_AVAILABLE,
    SCARD_E_NO_SERVICE,
    SCARD_S_SUCCESS,
    SCARD_SCOPE_USER,
    SCardEstablishContext,
    SCardListReaders,
    SCardReleaseContext,
)


def test_low_level():
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    assert hresult in (SCARD_S_SUCCESS, SCARD_E_NO_SERVICE)

    if hresult == SCARD_E_NO_SERVICE:
        return

    hresult, _ = SCardListReaders(hcontext, [])
    assert hresult in (SCARD_S_SUCCESS, SCARD_E_NO_READERS_AVAILABLE)

    # the computer we are using may not have a reader connected
    # so we can't do much

    hresult = SCardReleaseContext(hcontext)
    assert hresult == SCARD_S_SUCCESS



#! /usr/bin/env python3
"""
Simple panel that defines a dialog to send APDUs to a card.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import wx

from smartcard.util import toBytes, toHexString
from smartcard.wx.APDUHexValidator import APDUHexValidator
from smartcard.wx.SimpleSCardAppEventObserver import SimpleSCardAppEventObserver

[
    ID_TEXT_COMMAND,
    ID_TEXTCTRL_COMMAND,
    ID_TEXT_RESPONSE,
    ID_TEXTCTRL_RESPONSE,
    ID_TEXT_SW,
    ID_TEXT_SW1,
    ID_TEXTCTRL_SW1,
    ID_TEXT_SW2,
    ID_TEXTCTRL_SW2,
    ID_CARDSTATE,
    ID_TRANSMIT,
] = [wx.NewId() for x in range(11)]


class SampleAPDUManagerPanel(wx.Panel, SimpleSCardAppEventObserver):
    """A simple panel that displays activated cards and readers and can
    send APDU to a connected card."""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        SimpleSCardAppEventObserver.__init__(self)
        self.layoutControls()

        self.Bind(wx.EVT_BUTTON, self.OnTransmit, self.transmitbutton)

    # callbacks from SimpleSCardAppEventObserver interface
    def OnActivateCard(self, card):
        """Called when a card is activated by double-clicking
        on the card or reader tree control or toolbar.
        In this sample, we just connect to the card on the first activation."""
        SimpleSCardAppEventObserver.OnActivateCard(self, card)
        self.feedbacktext.SetLabel("Activated card: " + repr(card))
        self.transmitbutton.Enable()

    def OnActivateReader(self, reader):
        """Called when a reader is activated by double-clicking
        on the reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnActivateReader(self, reader)
        self.feedbacktext.SetLabel("Activated reader: " + repr(reader))
        self.transmitbutton.Disable()

    def OnDeactivateCard(self, card):
        """Called when a card is deactivated in the reader
        tree control or toolbar."""
        SimpleSCardAppEventObserver.OnActivateCard(self, card)
        self.feedbacktext.SetLabel("Deactivated card: " + repr(card))
        self.transmitbutton.Disable()

    def OnDeselectCard(self, card):
        """Called when a card is selected by clicking on the
        card or reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectCard(self, card)
        self.feedbacktext.SetLabel("Deselected card: " + repr(card))
        self.transmitbutton.Disable()

    def OnSelectCard(self, card):
        """Called when a card is selected by clicking on the
        card or reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectCard(self, card)
        self.feedbacktext.SetLabel("Selected card: " + repr(card))
        if hasattr(self.selectedcard, "connection"):
            self.transmitbutton.Enable()

    def OnSelectReader(self, reader):
        """Called when a reader is selected by clicking on the
        reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectReader(self, reader)
        self.feedbacktext.SetLabel("Selected reader: " + repr(reader))
        self.transmitbutton.Disable()

    # callbacks
    def OnTransmit(self, event):
        if hasattr(self.selectedcard, "connection"):
            apdu = self.commandtextctrl.GetValue()
            data, sw1, sw2 = self.selectedcard.connection.transmit(toBytes(apdu))
            self.SW1textctrl.SetValue("%x" % sw1)
            self.SW2textctrl.SetValue("%x" % sw2)
            self.responsetextctrl.SetValue(toHexString(data + [sw1, sw2]))
        event.Skip()

    def layoutControls(self):

        # create controls
        statictextCommand = wx.StaticText(
            self, ID_TEXT_COMMAND, "Command", wx.DefaultPosition, wx.DefaultSize, 0
        )
        self.commandtextctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_COMMAND,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_MULTILINE,
            validator=APDUHexValidator(),
        )
        statictextResponse = wx.StaticText(
            self, ID_TEXT_RESPONSE, "Response", wx.DefaultPosition, wx.DefaultSize, 0
        )
        self.responsetextctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_RESPONSE,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_MULTILINE | wx.TE_READONLY,
        )
        statictextStatusWords = wx.StaticText(
            self, ID_TEXT_SW, "Status Words", wx.DefaultPosition, wx.DefaultSize, 0
        )
        statictextSW1 = wx.StaticText(
            self, ID_TEXT_SW1, "SW1", wx.DefaultPosition, wx.DefaultSize, 0
        )
        self.SW1textctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_SW1,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_READONLY,
        )
        statictextSW2 = wx.StaticText(
            self, ID_TEXT_SW2, "SW2", wx.DefaultPosition, wx.DefaultSize, 0
        )
        self.SW2textctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_SW2,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_READONLY,
        )
        self.feedbacktext = wx.StaticText(
            self, ID_CARDSTATE, "", wx.DefaultPosition, wx.DefaultSize, 0
        )

        # layout controls
        boxsizerCommand = wx.BoxSizer(wx.HORIZONTAL)
        boxsizerCommand.Add(statictextCommand, 1, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerCommand.Add(self.commandtextctrl, 5, wx.EXPAND | wx.ALL, 5)

        boxsizerResponse = wx.BoxSizer(wx.HORIZONTAL)
        boxsizerResponse.Add(statictextResponse, 1, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerResponse.Add(self.responsetextctrl, 5, wx.EXPAND | wx.ALL, 5)

        boxsizerSW = wx.BoxSizer(wx.HORIZONTAL)
        boxsizerSW.Add(statictextSW1, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerSW.Add(self.SW1textctrl, 0, wx.EXPAND | wx.ALL, 5)
        boxsizerSW.Add(statictextSW2, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerSW.Add(self.SW2textctrl, 0, wx.EXPAND | wx.ALL, 5)

        item11 = wx.BoxSizer(wx.HORIZONTAL)
        item11.Add(statictextStatusWords, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        item11.Add(boxsizerSW, 0, wx.EXPAND | wx.ALL, 5)

        boxsizerResponseAndSW = wx.BoxSizer(wx.VERTICAL)
        boxsizerResponseAndSW.Add(boxsizerResponse, 0, wx.EXPAND | wx.ALL, 5)
        boxsizerResponseAndSW.Add(item11, 0, wx.EXPAND | wx.ALL, 5)

        staticboxAPDU = wx.StaticBox(self, -1, "APDU")
        boxsizerAPDU = wx.StaticBoxSizer(staticboxAPDU, wx.VERTICAL)
        boxsizerAPDU.Add(boxsizerCommand, 1, wx.EXPAND | wx.ALL, 5)
        boxsizerAPDU.Add(boxsizerResponseAndSW, 4, wx.EXPAND | wx.ALL, 5)

        staticboxEvents = wx.StaticBox(self, -1, "Card/Reader Events")
        boxsizerEvents = wx.StaticBoxSizer(staticboxEvents, wx.HORIZONTAL)
        boxsizerEvents.Add(self.feedbacktext, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        sizerboxTransmitButton = wx.BoxSizer(wx.HORIZONTAL)
        sizerboxTransmitButton.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        self.transmitbutton = wx.Button(
            self, ID_TRANSMIT, "Transmit", wx.DefaultPosition, wx.DefaultSize, 0
        )
        self.transmitbutton.Disable()
        sizerboxTransmitButton.Add(self.transmitbutton, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizerboxTransmitButton.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)

        sizerPanel = wx.BoxSizer(wx.VERTICAL)
        sizerPanel.Add(boxsizerAPDU, 3, wx.EXPAND | wx.ALL, 5)
        sizerPanel.Add(boxsizerEvents, 1, wx.EXPAND | wx.ALL, 5)
        sizerPanel.Add(sizerboxTransmitButton, 1, wx.EXPAND | wx.ALL, 5)

        self.SetSizer(sizerPanel)
        self.SetAutoLayout(True)
        sizerPanel.Fit(self)



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: Detect card insertion/removal

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import smartcard.util
from smartcard.scard import (
    INFINITE,
    SCARD_S_SUCCESS,
    SCARD_SCOPE_USER,
    SCARD_STATE_ATRMATCH,
    SCARD_STATE_CHANGED,
    SCARD_STATE_EMPTY,
    SCARD_STATE_EXCLUSIVE,
    SCARD_STATE_IGNORE,
    SCARD_STATE_INUSE,
    SCARD_STATE_MUTE,
    SCARD_STATE_PRESENT,
    SCARD_STATE_UNAVAILABLE,
    SCARD_STATE_UNAWARE,
    SCARD_STATE_UNKNOWN,
    SCardEstablishContext,
    SCardGetErrorMessage,
    SCardGetStatusChange,
    SCardListReaders,
    SCardReleaseContext,
    error,
)

srTreeATR = [0x3B, 0x77, 0x94, 0x00, 0x00, 0x82, 0x30, 0x00, 0x13, 0x6C, 0x9F, 0x22]
srTreeMask = [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]


def printstate(state):
    """Print card state"""
    reader, eventstate, atr = state
    print(reader + " " + smartcard.util.toHexString(atr, smartcard.util.HEX))
    if eventstate & SCARD_STATE_ATRMATCH:
        print("\tCard found")
    if eventstate & SCARD_STATE_UNAWARE:
        print("\tState unware")
    if eventstate & SCARD_STATE_IGNORE:
        print("\tIgnore reader")
    if eventstate & SCARD_STATE_UNAVAILABLE:
        print("\tReader unavailable")
    if eventstate & SCARD_STATE_EMPTY:
        print("\tReader empty")
    if eventstate & SCARD_STATE_PRESENT:
        print("\tCard present in reader")
    if eventstate & SCARD_STATE_EXCLUSIVE:
        print("\tCard allocated for exclusive use by another application")
    if eventstate & SCARD_STATE_INUSE:
        print("\tCard in used by another application but can be shared")
    if eventstate & SCARD_STATE_MUTE:
        print("\tCard is mute")
    if eventstate & SCARD_STATE_CHANGED:
        print("\tState changed")
    if eventstate & SCARD_STATE_UNKNOWN:
        print("\tState unknowned")


def main():
    """main"""
    try:
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to establish context: " + SCardGetErrorMessage(hresult))
        print("Context established!")

        try:
            hresult, readers = SCardListReaders(hcontext, [])
            if hresult != SCARD_S_SUCCESS:
                raise error("Failed to list readers: " + SCardGetErrorMessage(hresult))
            print("PCSC Readers:", readers)

            readerstates = []
            for reader in readers:
                readerstates += [(reader, SCARD_STATE_UNAWARE)]

            print("----- Current reader and card states are: -------")
            hresult, newstates = SCardGetStatusChange(hcontext, 0, readerstates)
            for i in newstates:
                printstate(i)

            print("----- Please insert or remove a card ------------")
            hresult, newstates = SCardGetStatusChange(hcontext, INFINITE, newstates)

            print("----- New reader and card states are: -----------")
            for i in newstates:
                printstate(i)

        finally:
            hresult = SCardReleaseContext(hcontext)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    "Failed to release context: " + SCardGetErrorMessage(hresult)
                )
            print("Released context.")

    except error as e:
        print(e)


if __name__ == "__main__":
    main()

    import sys

    if "win32" == sys.platform:
        print("press Enter to continue")
        sys.stdin.read(1)



"""Do not edit by hand"""
from smartcard.util import toHexString
expectedReaders = ['Gemalto PC Twin Reader', 'Gemalto PC Twin Reader 01']
expectedATRs = [[59, 167, 0, 64, 24, 128, 101, 162, 8, 1, 1, 82], [59, 111, 0, 0, 128, 90, 40, 19, 2, 16, 18, 43, 117, 13, 211, 130, 130, 144, 0]]
expectedATRinReader = {}
for i, reader in enumerate(expectedReaders):
    expectedATRinReader[reader] = expectedATRs[i]
expectedReaderForATR = {}
for i, reader in enumerate(expectedReaders):
    expectedReaderForATR[toHexString(expectedATRs[i])] = reader
expectedReaderGroups = ['SCard$DefaultReaders']



#! /usr/bin/env python3
"""Unit tests for smartcard.readers.

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import sys
import unittest

from smartcard import listReaders
from smartcard.scard import resourceManager
from smartcard.System import readergroups, readers

sys.path += [".."]

try:
    from local_config import expectedReaderGroups, expectedReaders
except ImportError:
    print("execute test suite first to generate the local_config.py file")
    sys.exit()


class testcase_readers(unittest.TestCase):
    """Test smartcard framework readers factory methods"""

    def testcase_enoughreaders(self):
        """enough readers"""
        self.assertTrue(len(readers()) > 1)

    def testcase_readers(self):
        """readers"""
        foundreaders = {}
        for reader in readers():
            foundreaders[str(reader)] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    def testcase_hashreaders(self):
        """hash readers"""
        foundreaders = {}
        for reader in readers():
            foundreaders[reader] = 1
        for reader in list(foundreaders.keys()):
            self.assertTrue(reader in readers())

    def testcase_legacyreaders(self):
        """legacy readers"""
        foundreaders = {}
        for reader in listReaders():
            foundreaders[reader] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    def testcase_readers_in_readergroup(self):
        """readers in readergroups"""
        foundreaders = {}
        for reader in readers(["SCard$DefaultReaders"]):
            foundreaders[str(reader)] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    def testcase_readers_in_readergroup_empty(self):
        """readers in readergroups empty"""
        foundreaders = {}
        for reader in readers([]):
            foundreaders[str(reader)] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    if "winscard" == resourceManager:

        def testcase_readers_in_readergroup_nonexistent(self):
            """readers in readergroups nonexistent"""
            foundreaders = {}
            for reader in readers(["dummy$group"]):
                foundreaders[reader] = 1
            for reader in expectedReaders:
                self.assertTrue(reader not in foundreaders)
            self.assertEqual(0, len(foundreaders))

    def testcase_readergroups(self):
        """readergroups"""
        foundreadergroups = {}
        # pylint: disable=not-an-iterable
        for readergroup in readergroups():
            foundreadergroups[readergroup] = 1
        for readergroup in expectedReaderGroups:
            self.assertTrue(readergroup in foundreadergroups)


if __name__ == "__main__":
    unittest.main(verbosity=1)



#! /usr/bin/env python3

# pylint: disable=invalid-name

"""
Sample script to illustrate toHexString() utility method

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys

from smartcard.util import COMMA, HEX, PACK, UPPERCASE, toHexString

print(40 * "-")
data = [59, 101, 0, 0, 156, 17, 1, 1, 3]
print("data = [59, 101, 0, 0, 156, 17, 1, 1, 3]")
print("toHexString(data) =", toHexString(data))
print("toHexString(data, COMMA) =", toHexString(data, COMMA))
print("toHexString(data, PACK) =", toHexString(data, PACK))
print("toHexString(data, HEX) =", toHexString(data, HEX))
print("toHexString(data, HEX | COMMA) =", toHexString(data, HEX | COMMA))
print("toHexString(data, HEX | UPPERCASE) =", toHexString(data, HEX | UPPERCASE))
print(
    "toHexString(data, HEX | UPPERCASE | COMMA) =",
    toHexString(data, HEX | UPPERCASE | COMMA),
)


print(40 * "-")
data = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
print("data = [ 0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03 ]")
print("toHexString(data, COMMA) =", toHexString(data, COMMA))
print("toHexString(data) =", toHexString(data))
print("toHexString(data, PACK) =", toHexString(data, PACK))
print("toHexString(data, HEX) =", toHexString(data, HEX))
print("toHexString(data, HEX | COMMA) =", toHexString(data, HEX | COMMA))
print("toHexString(data, HEX | UPPERCASE) =", toHexString(data, HEX | UPPERCASE))
print(
    "toHexString(data, HEX | UPPERCASE | COMMA) =",
    toHexString(data, HEX | UPPERCASE | COMMA),
)


if "win32" == sys.platform:
    print("press Enter to continue")
    sys.stdin.read(1)



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: send a Control Code to a card or
reader

__author__ = "Ludovic Rousseau"

Copyright 2009-2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys
import traceback

from smartcard.pcsc.PCSCExceptions import (
    BaseSCardException,
    EstablishContextException,
    ListReadersException,
    ReleaseContextException,
)
from smartcard.scard import (
    SCARD_CTL_CODE,
    SCARD_PROTOCOL_T0,
    SCARD_PROTOCOL_T1,
    SCARD_S_SUCCESS,
    SCARD_SCOPE_USER,
    SCARD_SHARE_SHARED,
    SCARD_UNPOWER_CARD,
    SCardConnect,
    SCardControl,
    SCardDisconnect,
    SCardEstablishContext,
    SCardListReaders,
    SCardReleaseContext,
    SCardTransmit,
    error,
)

# pylint: disable=missing-function-docstring


def can_do_verify_pin(hCard):
    FEATURE_VERIFY_PIN_DIRECT = 6
    return parse_get_feature_request(hCard, FEATURE_VERIFY_PIN_DIRECT)


def can_do_modify_pin(hCard):
    FEATURE_MODIFY_PIN_DIRECT = 7
    return parse_get_feature_request(hCard, FEATURE_MODIFY_PIN_DIRECT)


def parse_get_feature_request(hCard, feature):
    # check the reader can do a verify pin
    CM_IOCTL_GET_FEATURE_REQUEST = SCARD_CTL_CODE(3400)
    hresult, response = SCardControl(hCard, CM_IOCTL_GET_FEATURE_REQUEST, [])
    if hresult != SCARD_S_SUCCESS:
        raise BaseSCardException(hresult)
    print(response)
    while len(response) > 0:
        tag = response[0]
        if feature == tag:
            return (
                ((((response[2] << 8) + response[3]) << 8) + response[4]) << 8
            ) + response[5]
        response = response[6:]

    return None


def verifypin(hCard, control=None):
    if control is None:
        control = can_do_verify_pin(hCard)
        if control is None:
            raise BaseSCardException(message="Not a pinpad")

    command = [
        0x00,  # bTimerOut
        0x00,  # bTimerOut2
        0x82,  # bmFormatString
        0x08,  # bmPINBlockString
        0x00,  # bmPINLengthFormat
        0x08,
        0x04,  # wPINMaxExtraDigit
        0x02,  # bEntryValidationCondition
        0x01,  # bNumberMessage
        0x04,
        0x09,  # wLangId
        0x00,  # bMsgIndex
        0x00,
        0x00,
        0x00,  # bTeoPrologue
        13,
        0,
        0,
        0,  # ulDataLength
        0x00,
        0x20,
        0x00,
        0x00,
        0x08,
        0x30,
        0x30,
        0x30,
        0x30,
        0x30,
        0x30,
        0x30,
        0x30,
    ]  # abData
    hresult, response = SCardControl(hCard, control, command)
    if hresult != SCARD_S_SUCCESS:
        raise BaseSCardException(hresult)
    return hresult, response


def main():
    # pylint: disable=too-many-nested-blocks
    # pylint: disable=too-many-branches
    try:
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise EstablishContextException(hresult)
        print("Context established!")

        try:
            hresult, readers = SCardListReaders(hcontext, [])
            if hresult != SCARD_S_SUCCESS:
                raise ListReadersException(hresult)
            print("PCSC Readers:", readers)

            if len(readers) < 1:
                raise BaseSCardException(message="No smart card readers")

            for zreader in readers:

                print("Trying to Control reader:", zreader)

                try:
                    hresult, hcard, dwActiveProtocol = SCardConnect(
                        hcontext,
                        zreader,
                        SCARD_SHARE_SHARED,
                        SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1,
                    )
                    if hresult != SCARD_S_SUCCESS:
                        raise BaseSCardException(hresult)
                    print("Connected with active protocol", dwActiveProtocol)

                    try:
                        SELECT = [
                            0x00,
                            0xA4,
                            0x04,
                            0x00,
                            0x06,
                            0xA0,
                            0x00,
                            0x00,
                            0x00,
                            0x18,
                            0xFF,
                        ]
                        hresult, response = SCardTransmit(
                            hcard, dwActiveProtocol, SELECT
                        )
                        if hresult != SCARD_S_SUCCESS:
                            raise BaseSCardException(hresult)

                        cmd_verify = can_do_verify_pin(hcard)
                        if cmd_verify:
                            print(f"can do verify pin: 0x{cmd_verify:08X}")

                        cmd_modify = can_do_modify_pin(hcard)
                        if cmd_modify:
                            print(f"can do modify pin: 0x{cmd_modify:08X}")

                        hresult, response = verifypin(hcard, cmd_verify)
                        print("Control:", response)
                    except BaseSCardException as ex:
                        print("Exception:", ex)
                        traceback.print_exc()
                    finally:
                        hresult = SCardDisconnect(hcard, SCARD_UNPOWER_CARD)
                        if hresult != SCARD_S_SUCCESS:
                            raise BaseSCardException(hresult)
                        print("Disconnected")

                except BaseSCardException as ex:
                    print("SCard Exception:", ex)

        finally:
            hresult = SCardReleaseContext(hcontext)
            if hresult != SCARD_S_SUCCESS:
                raise ReleaseContextException(hresult)
            print("Released context.")

    except error as e:
        print(e)


if __name__ == "__main__":
    main()

    if "win32" == sys.platform:
        print("press Enter to continue")
        sys.stdin.read(1)



#! /usr/bin/env python3
"""
Sample for python PCSC wrapper module: List PCSC readers

__author__ = "https://www.gemalto.com/"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from smartcard.scard import (
    SCARD_S_SUCCESS,
    SCARD_SCOPE_USER,
    SCardEstablishContext,
    SCardGetErrorMessage,
    SCardListReaderGroups,
    SCardListReaders,
    SCardReleaseContext,
    error,
)

try:
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    if hresult != SCARD_S_SUCCESS:
        raise error("Failed to establish context: " + SCardGetErrorMessage(hresult))
    print("Context established!")

    try:
        hresult, readers = SCardListReaders(hcontext, [])
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to list readers: " + SCardGetErrorMessage(hresult))
        print("PCSC Readers:", readers)

        hresult, readerGroups = SCardListReaderGroups(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error(
                "Unable to list reader groups: " + SCardGetErrorMessage(hresult)
            )
        print("PCSC Reader groups:", readerGroups)

    finally:
        hresult = SCardReleaseContext(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error("Failed to release context: " + SCardGetErrorMessage(hresult))
        print("Released context.")

    import sys

    if "win32" == sys.platform:
        print("press Enter to continue")
        sys.stdin.read(1)

except error as e:
    print(e)
